/***********************************************************************
 *
 * HISTORY.C - NetSaint History CGI
 *
 * Written by Ethan Galstad (netsaint@linuxbox.com)
 * Last Modified: 08-10-1999
 *
 * This CGI program will display the history for the specified host.
 * If no host is specified, the history for all hosts will be displayed.
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 ***********************************************************************/

#include "../common/config.h"
#include "../common/locations.h"
#include "getcgi.h"
#include "cgiutils.h"
#include "../common/common.h"

#define DISPLAY_HOSTS			0

#define SERVICE_HISTORY			0
#define HOST_HISTORY			1

#define STATE_ALL			0
#define STATE_SOFT			1
#define STATE_HARD			2

void get_history(void);

void document_header(void);
void document_footer(void);
int process_cgivars(void);

extern char main_config_file[MAX_FILENAME_LENGTH];
extern char url_images_path[MAX_FILENAME_LENGTH];
extern char log_file[MAX_FILENAME_LENGTH];

int show_all_hosts=FALSE;
char host_name[MAX_HOSTNAME_LENGTH]="";
int display_type=DISPLAY_HOSTS;

int history_options=HISTORY_ALL;
int state_options=STATE_ALL;


int main(void){
	int error=FALSE;
	int result=OK;
	time_t t;
	char date_time[48];

	/* get the variables passed to us */
	error=process_cgivars();

	/* if there was an error with the variables, print out an error message */
	if(error==TRUE){
		document_header();
		printf("<p><div align=\"center\"><font color=\"red\"><strong>");
		printf("Invalid argument supplied!");
		printf("</strong></font></div></p>\n");
		document_footer();
	        }

	else{

		document_header();


		/* begin top table */
		printf("<p>\n");
		printf("<table border=0 width=100%%>\n");
		printf("<tr>\n");
		printf("<td align=left valign=top>\n");

		/* left column of the first row */
		printf("<table border=1>\n");
		printf("<tr><td bgcolor=%s>\n",HTML_STATUS_BG_EVEN_COLOR);
		printf("<font size=+2><b>Alert History</b></font><br>\n");
		printf("<font size=-2>\n");
		time(&t);
		get_time_string(&t,date_time,sizeof(date_time));
		printf("Last Updated: %s<BR>",date_time);
		printf("NetSaint Network Monitor - <a href=\"http://www.netsaint.org\" target=\"_new\">www.netsaint.org</a><br>\n");
		printf("</font>\n");
		printf("</td></tr>\n");

		if(show_all_hosts==FALSE){
			printf("<tr><td bgcolor=%s>\n",HTML_STATUS_BG_ODD_COLOR);
			printf("<font size=-2>\n");
			printf("<a href=\"history.cgi?host=all\">View history for all hosts</a>\n");
			printf("</font>\n");
			printf("</td></tr>\n");
	                }

		printf("</table>\n");
		printf("</td>\n");

		/* middle column of top row */
		printf("<td align=center valign=center>\n");
		printf("<font size=+2><b>");
		if(show_all_hosts==TRUE)
			printf("History For All Hosts\n");
		else
			printf("History For Host \"%s\"\n",host_name);
		printf("</b></font>\n");
		printf("</td>\n");


		/* right hand column of top row */
		printf("<td align=right valign=bottom>\n");

		printf("<table border=0>\n");
		printf("<form method=\"GET\" action=\"history.cgi\">\n");
		printf("<input type='hidden' name='host' value='%s'>\n",(show_all_hosts==TRUE)?"all":host_name);
		printf("<tr><td align=left>\n");
		printf("<font size=-2>\n");
		printf("State type options:<br>\n");
		printf("</font>\n");
		printf("<font size=-1>\n");
		printf("<select name='statetype'>\n");
		printf("<option value=%d %s>All state types\n",STATE_ALL,(state_options==STATE_ALL)?"selected":"");
		printf("<option value=%d %s>Soft states\n",STATE_SOFT,(state_options==STATE_SOFT)?"selected":"");
		printf("<option value=%d %s>Hard states\n",STATE_HARD,(state_options==STATE_HARD)?"selected":"");
		printf("</select>\n");
		printf("</font>\n");
		printf("</td></tr>\n");
		printf("<tr><td>\n");
		printf("<font size=-2>\n");
		printf("History detail level for %s host%s:<br>",(show_all_hosts==TRUE)?"all":"this",(show_all_hosts==TRUE)?"s":"");
		printf("</font>\n");
		printf("<font size=-1>\n");
		printf("<select name='type'>\n");
		printf("<option value=%d %s>All alerts\n",HISTORY_ALL,(history_options==HISTORY_ALL)?"selected":"");
		printf("<option value=%d %s>All service alerts\n",HISTORY_SERVICE_ALL,(history_options==HISTORY_SERVICE_ALL)?"selected":"");
		printf("<option value=%d %s>All host alerts\n",HISTORY_HOST_ALL,(history_options==HISTORY_HOST_ALL)?"selected":"");
		printf("<option value=%d %s>Service warning\n",HISTORY_SERVICE_WARNING,(history_options==HISTORY_SERVICE_WARNING)?"selected":"");
		printf("<option value=%d %s>Service unknown\n",HISTORY_SERVICE_UNKNOWN,(history_options==HISTORY_SERVICE_UNKNOWN)?"selected":"");
		printf("<option value=%d %s>Service critical\n",HISTORY_SERVICE_CRITICAL,(history_options==HISTORY_SERVICE_CRITICAL)?"selected":"");
		printf("<option value=%d %s>Service recovery\n",HISTORY_SERVICE_RECOVERY,(history_options==HISTORY_SERVICE_RECOVERY)?"selected":"");
		printf("<option value=%d %s>Host down\n",HISTORY_HOST_DOWN,(history_options==HISTORY_HOST_DOWN)?"selected":"");
		printf("<option value=%d %s>Host unreachable\n",HISTORY_HOST_UNREACHABLE,(history_options==HISTORY_HOST_UNREACHABLE)?"selected":"");
		printf("<option value=%d %s>Host recovery\n",HISTORY_HOST_RECOVERY,(history_options==HISTORY_HOST_RECOVERY)?"selected":"");
		printf("</select>\n");
		printf("<input type='submit' value='Update'>\n");
		printf("</font>\n");
		printf("</form>\n");
		printf("</font>\n");
		printf("</td></tr>\n");
		printf("</table>\n");

		printf("</td>\n");

		/* end of top table */
		printf("</tr>\n");
		printf("</table>\n");
		printf("</p>\n");


		reset_cgi_vars();

		result=read_cgi_config_file(DEFAULT_CGI_CONFIG_FILE);
		if(result==ERROR){
			printf("<p><div align=\"center\"><font color=\"red\"><strong>\n");
			printf("Error: Could not open CGI configuration file '%s' for reading!\n",DEFAULT_CGI_CONFIG_FILE);
			printf("</strong></font></div></p>\n");
			document_footer();
			return ERROR;
	                }

		result=read_main_config_file(main_config_file);
		if(result==ERROR){
			printf("<p><div align=\"center\"><font color=\"red\"><strong>\n");
			printf("Error: Could not open main configuration file for reading!\n");
			printf("</strong></font></div></p>\n");
			document_footer();
			return ERROR;
	                }

		get_history();

		document_footer();
	        }

	return OK;
        }


void get_history(void){
	FILE *fp;
	char image[MAX_INPUT_BUFFER];
	char image_alt[MAX_INPUT_BUFFER];
	char input_buffer[MAX_INPUT_BUFFER];
	char match1[MAX_INPUT_BUFFER];
	char match2[MAX_INPUT_BUFFER];
	int found_line=FALSE;
	int system_message=FALSE;
	int display_line=FALSE;
	time_t t;
	char date_time[48];
	char *temp_buffer;
	int history_type;
	int history_detail_type;

	fp=fopen(log_file,"r");
	if(fp==NULL){
		printf("<HR>\n");
		printf("<p><div align=\"center\"><font color=\"red\"><strong>");
		printf("Error: Cannot open log file for reading!");
		printf("</strong></font></div></p>\n");
		printf("<HR>\n");
		return;
	        }

	printf("<HR>\n");

	printf("<P>\n");
	printf("<font size=-1>\n");

	while(fgets(input_buffer,MAX_INPUT_BUFFER-1,fp)){

		strcpy(image,"");
		strcpy(image_alt,"");
		system_message=FALSE;

		if(strstr(input_buffer,"SERVICE ALERT:")){
			
			history_type=SERVICE_HISTORY;

			if(strstr(input_buffer,";CRITICAL;")){
				strcpy(image,CRITICAL_ICON);
				strcpy(image_alt,CRITICAL_ICON_ALT);
				history_detail_type=HISTORY_SERVICE_CRITICAL;
                                }
			else if(strstr(input_buffer,";WARNING;")){
				strcpy(image,WARNING_ICON);
				strcpy(image_alt,WARNING_ICON_ALT);
				history_detail_type=HISTORY_SERVICE_WARNING;
                                }
			else if(strstr(input_buffer,";UNKNOWN;")){
				strcpy(image,UNKNOWN_ICON);
				strcpy(image_alt,UNKNOWN_ICON_ALT);
 				history_detail_type=HISTORY_SERVICE_UNKNOWN;
                               }
			else if(strstr(input_buffer,";RECOVERY;")){
				strcpy(image,RECOVERY_ICON);
				strcpy(image_alt,RECOVERY_ICON_ALT);
				history_detail_type=HISTORY_SERVICE_RECOVERY;
                                }
		        }

		else if(strstr(input_buffer,"HOST ALERT:")){

			history_type=HOST_HISTORY;

			if(strstr(input_buffer,";DOWN")){
				strcpy(image,HOST_CRITICAL_ICON);
				strcpy(image_alt,HOST_CRITICAL_ICON_ALT);
				history_detail_type=HISTORY_HOST_DOWN;
		                }
			else if(strstr(input_buffer,";UNREACHABLE")){
				strcpy(image,HOST_CRITICAL_ICON);
				strcpy(image_alt,HOST_CRITICAL_ICON_ALT);
				history_detail_type=HISTORY_HOST_UNREACHABLE;
		                }
			else if(strstr(input_buffer,";RECOVERY")){
				strcpy(image,HOST_RECOVERY_ICON);
				strcpy(image_alt,HOST_RECOVERY_ICON_ALT);
				history_detail_type=HISTORY_HOST_RECOVERY;
		                }
		        }

		else if(strstr(input_buffer," starting...")){
			strcpy(image,START_ICON);
			strcpy(image_alt,START_ICON_ALT);
			system_message=TRUE;
		        }
		else if(strstr(input_buffer," shutting down...")){
			strcpy(image,STOP_ICON);
			strcpy(image_alt,STOP_ICON_ALT);
			system_message=TRUE;
		        }
		else if(strstr(input_buffer," restarting...")){
			strcpy(image,RESTART_ICON);
			strcpy(image_alt,RESTART_ICON_ALT);
			system_message=TRUE;
		        }

		/* get the timestamp */
		temp_buffer=strtok(input_buffer,"]");
		t=strtoul(temp_buffer+1,NULL,10);
		get_time_string(&t,date_time,sizeof(date_time));
		strip(date_time);

		temp_buffer=strtok(NULL,"\n");

		if(strcmp(image,"")){

			display_line=FALSE;

			if(system_message==TRUE)
				display_line=TRUE;

			else if(display_type==DISPLAY_HOSTS){

				sprintf(match1," HOST ALERT: %s;",host_name);
				sprintf(match2," SERVICE ALERT: %s;",host_name);

				if(show_all_hosts==TRUE)
					display_line=TRUE;
				else if(strstr(temp_buffer,match1))
					display_line=TRUE;
				else if(strstr(temp_buffer,match2))
					display_line=TRUE;

				if(display_line==TRUE){
					if(history_options==HISTORY_ALL)
						display_line=TRUE;
					else if(history_options==HISTORY_HOST_ALL && history_type==HOST_HISTORY)
						display_line=TRUE;
					else if(history_options==HISTORY_SERVICE_ALL && history_type==SERVICE_HISTORY)
						display_line=TRUE;
					else if(history_detail_type & history_options)
						display_line=TRUE;
					else 
						display_line=FALSE;
			                }

				if(display_line==TRUE){
					if(state_options==STATE_ALL)
						display_line=TRUE;
					else if((state_options & STATE_SOFT) && strstr(temp_buffer,";SOFT;"))
						display_line=TRUE;
					else if((state_options & STATE_HARD) && strstr(temp_buffer,";HARD;"))
						display_line=TRUE;
					else
						display_line=FALSE;
				        }
			        }
			
			if(display_line==TRUE){
				printf("<img align=\"left\" src=\"%s%s\" alt=\"%s\">[%s] %s<br clear=\"all\">\n",url_images_path,image,image_alt,date_time,temp_buffer);
				found_line=TRUE;
			        }
		        }

                }

	if(found_line==FALSE){
		printf("<div align=\"center\"><font color=\"red\"><strong>");
		printf("No history is available yet");
		printf("</strong></font></div>\n");
	        }

	printf("</FONT>\n");
	printf("</P>\n");
	
	printf("<HR>\n");

	fclose(fp);

	return;
        }


void document_header(void){

	printf("Content-type: text/html\r\n\r\n");

	printf("<html>\n");
	printf("<head>\n");
	printf("<META HTTP-EQUIV=\"Pragma\" CONTENT=\"no-cache\">\n");
	printf("<title>\n");
	printf("NetSaint History\n");
	printf("</title>\n");
	printf("</head>\n");
	printf("<body>\n");

	return;
        }


void document_footer(void){

	printf("</body>\n");
	printf("</html>\n");

	return;
        }


int process_cgivars(void){
	char **variables;
	int error=FALSE;
	int x;

	variables=getcgivars();

	for(x=0;variables[x]!=NULL;x++){

		/* do some basic length checking on the variable identifier to prevent buffer overflows */
		if(strlen(variables[x])>=MAX_INPUT_BUFFER-1){
			x++;
			continue;
		        }

		/* we found the host argument */
		else if(!strcmp(variables[x],"host")){
			x++;
			if(variables[x]==NULL){
				error=TRUE;
				break;
			        }

			/* protect against buffer overflows */
			if(strlen(variables[x])>=MAX_HOSTNAME_LENGTH-1){
				strcpy(host_name,"all");
			        }
			else
				strcpy(host_name,variables[x]);

			display_type=DISPLAY_HOSTS;

			if(!strcmp(host_name,"all"))
				show_all_hosts=TRUE;
			else
				show_all_hosts=FALSE;
		        }
	
	
		/* we found the history type argument */
		else if(!strcmp(variables[x],"type")){
			x++;
			if(variables[x]==NULL){
				error=TRUE;
				break;
			        }

			history_options=atoi(variables[x]);
		        }
	
		/* we found the history state type argument */
		else if(!strcmp(variables[x],"statetype")){
			x++;
			if(variables[x]==NULL){
				error=TRUE;
				break;
			        }

			state_options=atoi(variables[x]);
		        }
	
	        }

	return error;
        }

