/************************************************************************
 *
 * NOTIFICATIONS.C - NetSaint Notifications CGI
 *
 * Written by Ethan Galstad (netsaint@linuxbox.com)
 * Last Modified: 08-10-1999
 *
 * This CGI program will display the notification events for 
 * a given host or contact or for all contacts/hosts.
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 ***********************************************************************/

#include "../common/config.h"
#include "../common/locations.h"
#include "getcgi.h"
#include "cgiutils.h"
#include "../common/common.h"

#define FIND_HOST		1
#define FIND_CONTACT		2

#define MAX_QUERYNAME_LENGTH	256

#define SERVICE_NOTIFICATION	0
#define HOST_NOTIFICATION	1

#define SERVICE_NOTIFICATION_STRING 	"SERVICE NOTIFICATION"
#define HOST_NOTIFICATION_STRING	"HOST NOTIFICATION"

void get_notifications(int,char *);

void document_header();
void document_footer();
int process_cgivars();

extern char main_config_file[MAX_FILENAME_LENGTH];
extern char url_html_path[MAX_FILENAME_LENGTH];
extern char url_docs_path[MAX_FILENAME_LENGTH];
extern char log_file[MAX_FILENAME_LENGTH];

int find_type=FIND_HOST;
int find_all=FALSE;
char find_query[MAX_QUERYNAME_LENGTH]="";

int notification_options=NOTIFICATION_ALL;


int main(void){
	int error=FALSE;
	int result=OK;
	time_t t;
	char date_time[48];

	error=process_cgivars();

	document_header();

	if(error==FALSE){

		/* begin top table */
		printf("<p>\n");
		printf("<table border=0 width=100%%>\n");
		printf("<tr>\n");
		printf("<td align=left valign=top>\n");

		/* left column of the first row */
		printf("<table border=1>\n");
		printf("<tr><td bgcolor=%s>\n",HTML_STATUS_BG_EVEN_COLOR);
		printf("<font size=+2><b>Notifications</b></font><br>\n");
		printf("<font size=-2>\n");
		time(&t);
		get_time_string(&t,date_time,sizeof(date_time));
		printf("Last Updated: %s<BR>",date_time);
		printf("NetSaint Network Monitor - <a href=\"http://www.netsaint.org\" target=\"_new\">www.netsaint.org</a><br>\n");
		printf("</font>\n");
		printf("</td></tr>\n");

		if(find_all==FALSE){
			printf("<tr><td bgcolor=%s>\n",HTML_STATUS_BG_ODD_COLOR);
			printf("<font size=-2>\n");
			printf("<a href=\"notifications.cgi?%s=all\">View notifications for all %s</a>\n",(find_type==FIND_HOST)?"host":"contact",(find_type==FIND_HOST)?"hosts":"contacts");
			printf("</font>\n");
			printf("</td></tr>\n");
	                }

		printf("</table>\n");
		printf("</td>\n");

		/* middle column of top row */
		printf("<td align=center valign=bottom>\n");
		printf("<font size=+2><b>");
		if(find_all==TRUE)
			printf("Notifications For All %s\n",(find_type==FIND_HOST)?"Hosts":"Contacts");
		else
			printf("Notifications For %s \"%s\"",(find_type==FIND_HOST)?"Host":"Contact",find_query);
		printf("</b></font>\n");
		printf("</td>\n");


		/* right hand column of top row */
		printf("<td align=right valign=bottom>\n");

		printf("<table border=0>\n");
		printf("<tr><td align=left>\n");
		printf("<font size=-2>\n");
		printf("Notification detail level for %s %s%s:<br>",(find_all==TRUE)?"all":"this",(find_type==FIND_HOST)?"host":"contact",(find_all==TRUE)?"s":"");
		printf("</font>\n");
		printf("</td></tr>\n");
		printf("<tr><td>\n");
		printf("<form method=\"GET\" action=\"notifications.cgi\">\n");
		printf("<input type='hidden' name='%s' value='%s'>\n",(find_type==FIND_HOST)?"host":"contact",find_query);
		printf("<font size=-1>\n");
		printf("<select name='type'>\n");
		printf("<option value=%d %s>All notifications\n",NOTIFICATION_ALL,(notification_options==NOTIFICATION_ALL)?"selected":"");
		printf("<option value=%d %s>All service notifications\n",NOTIFICATION_SERVICE_ALL,(notification_options==NOTIFICATION_SERVICE_ALL)?"selected":"");
		printf("<option value=%d %s>All host notifications\n",NOTIFICATION_HOST_ALL,(notification_options==NOTIFICATION_HOST_ALL)?"selected":"");
		printf("<option value=%d %s>Service warning\n",NOTIFICATION_SERVICE_WARNING,(notification_options==NOTIFICATION_SERVICE_WARNING)?"selected":"");
		printf("<option value=%d %s>Service unknown\n",NOTIFICATION_SERVICE_UNKNOWN,(notification_options==NOTIFICATION_SERVICE_UNKNOWN)?"selected":"");
		printf("<option value=%d %s>Service critical\n",NOTIFICATION_SERVICE_CRITICAL,(notification_options==NOTIFICATION_SERVICE_CRITICAL)?"selected":"");
		printf("<option value=%d %s>Service recovery\n",NOTIFICATION_SERVICE_RECOVERY,(notification_options==NOTIFICATION_SERVICE_RECOVERY)?"selected":"");
		printf("<option value=%d %s>Host down\n",NOTIFICATION_HOST_DOWN,(notification_options==NOTIFICATION_HOST_DOWN)?"selected":"");
		printf("<option value=%d %s>Host unreachable\n",NOTIFICATION_HOST_UNREACHABLE,(notification_options==NOTIFICATION_HOST_UNREACHABLE)?"selected":"");
		printf("<option value=%d %s>Host recovery\n",NOTIFICATION_HOST_RECOVERY,(notification_options==NOTIFICATION_HOST_RECOVERY)?"selected":"");
		printf("</select>\n");
		printf("<input type='submit' value='Update'>\n");
		printf("</form>\n");
		printf("</font>\n");
		printf("</td></tr>\n");
		printf("</table>\n");

		printf("</td>\n");

		/* end of top table */
		printf("</tr>\n");
		printf("</table>\n");
		printf("</p>\n");


		reset_cgi_vars();

		result=read_cgi_config_file(DEFAULT_CGI_CONFIG_FILE);
		if(result==ERROR){
			printf("<p><div align=\"center\"><font color=\"red\"><strong>\n");
			printf("Error: Could not open CGI configuration file '%s' for reading!\n",DEFAULT_CGI_CONFIG_FILE);
			printf("</strong></font></div></p>\n");
			document_footer();
			return ERROR;
	                }

		result=read_main_config_file(main_config_file);
		if(result==ERROR){
			printf("<p><div align=\"center\"><font color=\"red\"><strong>\n");
			printf("Error: Could not open main configuration file for reading!\n");
			printf("</strong></font></div></p>\n");
			document_footer();
			return ERROR;
	                }

		get_notifications(find_type,find_query);
	        }

	else{
		printf("<p><div align=\"center\"><font color=\"red\"><strong>");
		printf("Invalid argument supplied");
		printf("</strong></font></div></p>\n"); 
	        }

	document_footer();
	
	return OK;
        }


void get_notifications(int find_type,char *varvalue){
	FILE *fp;
	char input_buffer[MAX_INPUT_BUFFER];
	char *temp_buffer;
	char date_time[MAX_INPUT_BUFFER];
	char alert_level[MAX_INPUT_BUFFER];
	char contact_name[MAX_INPUT_BUFFER];
	char service_name[MAX_INPUT_BUFFER];
	char host_name[MAX_INPUT_BUFFER];
	char method_name[MAX_INPUT_BUFFER];
	char alert_color[MAX_INPUT_BUFFER];
	int show_entry;
	int total_notifications;
	int notification_type;
	int notification_detail_type;
	int odd=0;
	time_t t;

	fp=fopen(log_file,"r");
	if(fp==NULL){
		printf("<p><div align=\"center\"><font color=\"red\"><strong>");
		printf("Error: Cannot open log file for reading!");
		printf("</strong></font></div></p>\n");
		return;
	        }

	total_notifications=0;
  
	while(fgets(input_buffer,MAX_INPUT_BUFFER-1,fp)){
    
		/* see if this line contains the notification event string */
		if(strstr(input_buffer,HOST_NOTIFICATION_STRING)||strstr(input_buffer,SERVICE_NOTIFICATION_STRING)){

			if(total_notifications==0){
				printf("<p>\n");
				printf("<div align=\"center\">\n");
				printf("<table border=0 cellspacing=3 cellpadding=3>\n");
				printf("<tr>\n");
				printf("<td bgcolor=\"%s\"><FONT COLOR=\"%s\"><B>Host</B></FONT></td>\n",HTML_NOTIFICATION_BG_COLOR,HTML_NOTIFICATION_FG_COLOR);
				printf("<td bgcolor=\"%s\"><FONT COLOR=\"%s\"><B>Service</B></FONT></td>\n",HTML_NOTIFICATION_BG_COLOR,HTML_NOTIFICATION_FG_COLOR);
				printf("<td bgcolor=\"%s\"><FONT COLOR=\"%s\"><B>Status</B></FONT></td>\n",HTML_NOTIFICATION_BG_COLOR,HTML_NOTIFICATION_FG_COLOR);
				printf("<td bgcolor=\"%s\"><FONT COLOR=\"%s\"><B>Date/Time</B></FONT></td>\n",HTML_NOTIFICATION_BG_COLOR,HTML_NOTIFICATION_FG_COLOR);
				printf("<td bgcolor=\"%s\"><FONT COLOR=\"%s\"><B>Contact</B></FONT></td>\n",HTML_NOTIFICATION_BG_COLOR,HTML_NOTIFICATION_FG_COLOR);
				printf("<td bgcolor=\"%s\"><FONT COLOR=\"%s\"><B>Notification Command</B></FONT></td>\n",HTML_NOTIFICATION_BG_COLOR,HTML_NOTIFICATION_FG_COLOR);
				printf("<td bgcolor=\"%s\"><FONT COLOR=\"%s\"><B>Information</B></FONT></td>\n",HTML_NOTIFICATION_BG_COLOR,HTML_NOTIFICATION_FG_COLOR);
				printf("</tr>\n");
			        }

			total_notifications++;

			if(strstr(input_buffer,HOST_NOTIFICATION_STRING))
				notification_type=HOST_NOTIFICATION;
			else
				notification_type=SERVICE_NOTIFICATION;
      
			/* get the date/time */
			temp_buffer=(char *)strtok(input_buffer,"]");
			t=strtoul(temp_buffer+1,NULL,10);
			get_time_string(&t,date_time,sizeof(date_time));
			strip(date_time);

			/* get the contact name */
			temp_buffer=(char *)strtok(NULL,":");
			temp_buffer=(char *)strtok(NULL,";");
			sprintf(contact_name,"%s",(temp_buffer==NULL)?"":temp_buffer+1);

			/* get the host name */
			temp_buffer=(char *)strtok(NULL,";");
			sprintf(host_name,"%s",(temp_buffer==NULL)?"":temp_buffer);

			/* get the service name */
			if(notification_type==SERVICE_NOTIFICATION){
				temp_buffer=(char *)strtok(NULL,";");
				sprintf(service_name,"%s",(temp_buffer==NULL)?"":temp_buffer);
			        }

			/* get the alert level */
			temp_buffer=(char *)strtok(NULL,";");
			sprintf(alert_level,"%s",(temp_buffer==NULL)?"":temp_buffer);

			if(notification_type==SERVICE_NOTIFICATION){

				if(!strcmp(alert_level,"CRITICAL")){
					strcpy(alert_color,CRITICAL_COLOR);
					notification_detail_type=NOTIFICATION_SERVICE_CRITICAL;
				        }
				else if(!strcmp(alert_level,"WARNING")){
					strcpy(alert_color,WARNING_COLOR);
					notification_detail_type=NOTIFICATION_SERVICE_WARNING;
				        }
				else if(!strcmp(alert_level,"RECOVERY")){
					strcpy(alert_color,RECOVERY_COLOR);
					notification_detail_type=NOTIFICATION_SERVICE_RECOVERY;
				        }
				else{
					strcpy(alert_color,UNKNOWN_COLOR);
					notification_detail_type=NOTIFICATION_SERVICE_UNKNOWN;
				        }
			        }

			else{

				if(!strcmp(alert_level,"DOWN")){
					strcpy(alert_color,HOST_DOWN_COLOR);
					strcpy(alert_level,"HOST DOWN");
					notification_detail_type=NOTIFICATION_HOST_DOWN;
				        }
				if(!strcmp(alert_level,"UNREACHABLE")){
					strcpy(alert_color,HOST_UNREACHABLE_COLOR);
					strcpy(alert_level,"HOST UNREACHABLE");
					notification_detail_type=NOTIFICATION_HOST_UNREACHABLE;
				        }
				else if(!strcmp(alert_level,"RECOVERY")){
					strcpy(alert_color,HOST_RECOVERY_COLOR);
					strcpy(alert_level,"HOST RECOVERY");
					notification_detail_type=NOTIFICATION_HOST_RECOVERY;
				        }
			        }

			/* get the method name */
			temp_buffer=(char *)strtok(NULL,";");
			sprintf(method_name,"%s",(temp_buffer==NULL)?"":temp_buffer);

			/* move to the informational message */
			temp_buffer=strtok(NULL,";");

			show_entry=FALSE;
      
			/* if we're searching by contact, filter out unwanted contact */
			if(find_type==FIND_CONTACT){
				if(find_all==TRUE)
					show_entry=TRUE;
				else if(!strcmp(varvalue,contact_name))
					show_entry=TRUE;
			        }

			if(find_type==FIND_HOST){
				if(find_all==TRUE)
					show_entry=TRUE;
				else if(!strcmp(varvalue,host_name))
					show_entry=TRUE;
			        }

			if(show_entry==TRUE){
				if(notification_options==NOTIFICATION_ALL)
					show_entry=TRUE;
				else if(notification_options==NOTIFICATION_HOST_ALL && notification_type==HOST_NOTIFICATION)
					show_entry=TRUE;
				else if(notification_options==NOTIFICATION_SERVICE_ALL && notification_type==SERVICE_NOTIFICATION)
					show_entry=TRUE;
				else if(notification_detail_type & notification_options)
					show_entry=TRUE;
				else 
					show_entry=FALSE;
			        }

			if(show_entry==TRUE){
				if(odd){
					odd=0;
					printf("<tr bgcolor=%s>\n",HTML_NOTIFICATION_BG_ODD_COLOR);
				        }
				else{
					odd=1;
					printf("<tr bgcolor=%s>\n",HTML_NOTIFICATION_BG_EVEN_COLOR);
				        }
				printf("<td><a href=\"%shosts.html#%s\"><font size=-1>%s</font></a></td>\n",url_html_path,host_name,host_name);
				if(notification_type==SERVICE_NOTIFICATION)
					printf("<td><a href=\"%sservices.html#%s;%s\"><font size=-1>%s</font></a></td>\n",url_html_path,host_name,service_name,service_name);
				else
					printf("<td><font size=-1>N/A</font></td>\n");
				printf("<td bgcolor=\"%s\"><a href=\"%sstatuslevels.html#%s\"><font size=-1>%s</font></a></td>\n",alert_color,url_docs_path,alert_level,alert_level);
				printf("<td><font size=-1>%s</font></td>\n",date_time);
				printf("<td><a href=\"%scontacts.html#%s\"><font size=-1>%s</font></a></td>\n",url_html_path,contact_name,contact_name);
				printf("<td><a href=\"%scommands.html#%s\"><font size=-1>%s</font></a></td>\n",url_html_path,method_name,method_name);
				printf("<td><font size=-1>%s</font></td>\n",temp_buffer);
				printf("</tr>\n");
			        }
		        }
	        }


	if(total_notifications==0){
		printf("<p><div align=\"center\"><font color=\"red\"><strong>");
		printf("No notifications have been logged yet");
		printf("</strong></font></div></p>\n");
	        }
	else{
		printf("</table>\n");
		printf("</div>\n");
		printf("</p>\n");
	        }

	fclose(fp);

	return;
        }



void document_header(void){

	printf("Content-type: text/html\r\n\r\n");

	printf("<html>\n");
	printf("<head>\n");
	printf("<META HTTP-EQUIV=\"Pragma\" CONTENT=\"no-cache\">\n");
	printf("<title>\n");
	printf("NetSaint Notifications\n");
	printf("</title>\n");
	printf("</head>\n");
	printf("<body>\n");

	return;
        }


void document_footer(void){

	printf("</body>\n");
	printf("</html>\n");

	return;
        }


int process_cgivars(void){
	char **variables;
	int error=FALSE;
	int x;

	variables=getcgivars();

	for(x=0;variables[x]!=NULL;x++){

		/* do some basic length checking on the variable identifier to prevent buffer overflows */
		if(strlen(variables[x])>=MAX_INPUT_BUFFER-1){
			x++;
			continue;
		        }

		/* we found the host argument */
		else if(!strcmp(variables[x],"host")){
			find_type=FIND_HOST;
			x++;
			if(variables[x]==NULL){
				error=TRUE;
				break;
			        }

			/* protect against buffer overflows */
			if(strlen(variables[x])>=MAX_QUERYNAME_LENGTH-1){
				strcpy(find_query,"all");
			        }
			else
				strcpy(find_query,variables[x]);

			if(!strcmp(find_query,"all"))
				find_all=TRUE;
			else
				find_all=FALSE;
		        }
	
		/* we found the contact argument */
		else if(!strcmp(variables[x],"contact")){
			find_type=FIND_CONTACT;
			x++;
			if(variables[x]==NULL){
				error=TRUE;
				break;
			        }

			/* protect against buffer overflows */
			if(strlen(variables[x])>=MAX_QUERYNAME_LENGTH-1){
				strcpy(find_query,"all");
			        }
			else
				strcpy(find_query,variables[x]);

			if(!strcmp(find_query,"all"))
				find_all=TRUE;
			else
				find_all=FALSE;
		        }
	
		/* we found the notification type argument */
		else if(!strcmp(variables[x],"type")){
			x++;
			if(variables[x]==NULL){
				error=TRUE;
				break;
			        }

			notification_options=atoi(variables[x]);
		        }
	
	        }

	return error;
        }

